<?php

declare(strict_types=1);

namespace Dedi\SyliusAdvertisingPlugin\Entity;

use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\Common\Collections\Collection;
use Doctrine\ORM\Mapping as ORM;
use Gedmo\Mapping\Annotation as Gedmo;
use Sylius\Component\Core\Model\ChannelInterface;
use Symfony\Component\Validator\Constraints as Assert;

/**
 * @ORM\Entity()
 * @ORM\Table(name="dedi_advertising_zone")
 */
class AdvertisingZone implements AdvertisingZoneInterface
{
    /**
     * @ORM\Id()
     * @ORM\GeneratedValue(strategy="AUTO")
     * @ORM\Column(type="integer", nullable=false, name="id")
     */
    protected ?int $id = null;

    /**
     * @ORM\Column(type="boolean", nullable=false, name="zone_enabled")
     */
    protected bool $enabled = true;

    /**
     * @ORM\Column(type="string", nullable=false, name="zone_code")
     * @Gedmo\Slug(fields={"id"}, unique=true)
     * @Assert\NotBlank(groups={"default", "sylius"})
     */
    protected ?string $code = null;

    /**
     * @ORM\ManyToMany(targetEntity="Dedi\SyliusAdvertisingPlugin\Entity\Advertising")
     * @ORM\JoinTable(name="dedi_advertising_zone_advertising",
     *     joinColumns={@ORM\JoinColumn(name="zone_id", referencedColumnName="id")},
     *     inverseJoinColumns={@ORM\JoinColumn(name="advertising_id", referencedColumnName="id")}
     * )
     */
    protected Collection $advertising;

    /**
     * @ORM\ManyToMany(targetEntity="Sylius\Component\Core\Model\ChannelInterface")
     * @ORM\JoinTable(name="dedi_advertising_zone_channels",
     *     joinColumns={@ORM\JoinColumn(name="zone_id", referencedColumnName="id")},
     *     inverseJoinColumns={@ORM\JoinColumn(name="channel_id", referencedColumnName="id")}
     * )
     */
    protected Collection $channels;

    public function __construct()
    {
        $this->advertising = new ArrayCollection();
        $this->channels = new ArrayCollection();
    }

    public function getId(): ?int
    {
        return $this->id;
    }

    public function isEnabled(): bool
    {
        return $this->enabled;
    }

    public function setEnabled(bool $enabled): self
    {
        $this->enabled = $enabled;

        return $this;
    }

    public function getCode(): string
    {
        return (string) $this->code;
    }

    public function setCode(?string $code): self
    {
        $this->code = $code;

        return $this;
    }

    public function getAdvertising(): Collection
    {
        return $this->advertising;
    }

    public function hasAdvertising(): bool
    {
        return !$this->advertising->isEmpty();
    }

    public function addAdvertising(AdvertisingInterface $advertising): bool
    {
        if (!$this->advertising->contains($advertising)) {
            return $this->advertising->add($advertising);
        }

        return false;
    }

    public function removeAdvertising(AdvertisingInterface $advertising): bool
    {
        return $this->advertising->removeElement($advertising);
    }

    public function getChannels(): Collection
    {
        return $this->channels;
    }

    public function setChannels(Collection $channels): self
    {
        $this->channels = $channels;

        return $this;
    }

    public function addChannel(ChannelInterface $channel): bool
    {
        if (!$this->channels->contains($channel)) {
            return $this->channels->add($channel);
        }

        return false;
    }

    public function removeChannel(ChannelInterface $channel): bool
    {
        return $this->channels->removeElement($channel);
    }
}
