<?php

declare(strict_types=1);

namespace Dedi\SyliusAdvertisingPlugin\Form;

use Dedi\SyliusAdvertisingPlugin\Entity\AdvertisingImage;
use Sylius\Bundle\CoreBundle\Form\Type\ImageType;
use Sylius\Component\Core\Uploader\ImageUploaderInterface;
use Symfony\Component\Form\Extension\Core\Type\FileType;
use Symfony\Component\Form\FormBuilderInterface;
use Symfony\Component\Form\FormEvent;
use Symfony\Component\Form\FormEvents;

class AdvertisingImageType extends ImageType
{
    private ImageUploaderInterface $uploader;

    public function __construct(ImageUploaderInterface $uploader)
    {
        parent::__construct(AdvertisingImage::class, ['default', 'sylius']);

        $this->uploader = $uploader;
    }

    public function buildForm(FormBuilderInterface $builder, array $options): void
    {
        $builder
            ->remove('type')
            ->add('file', FileType::class, [
                'label' => 'dedi_advertising.form.advertising.file',
            ])
        ;

        $builder->addEventListener(FormEvents::POST_SUBMIT, function (FormEvent $event) {
            $image = $event->getData();
            if (null != $image) {
                $this->uploader->upload($image);
            }
        });
    }

    public function getBlockPrefix(): string
    {
        return 'dedi_advertising_image';
    }
}
