<?php

declare(strict_types=1);

namespace Dedi\SyliusAdvertisingPlugin\Twig;

use Dedi\SyliusAdvertisingPlugin\Repository\AdvertisingZoneRepositoryInterface;
use Sylius\Component\Channel\Context\ChannelContextInterface;
use Sylius\Component\Core\Model\ChannelInterface;
use Sylius\Component\Locale\Context\LocaleContextInterface;
use Symfony\Component\Templating\EngineInterface;
use Twig\Extension\AbstractExtension;
use Twig\TwigFunction;

class AdvertisingExtension extends AbstractExtension
{
    private AdvertisingZoneRepositoryInterface $advertisingZoneRepository;

    private EngineInterface $engine;

    private LocaleContextInterface $localeContext;

    private ChannelContextInterface $channelContext;

    public function __construct(
        AdvertisingZoneRepositoryInterface $advertisingZoneRepository,
        ChannelContextInterface $channelContext,
        LocaleContextInterface $localeContext,
        EngineInterface $engine)
    {
        $this->advertisingZoneRepository = $advertisingZoneRepository;
        $this->channelContext = $channelContext;
        $this->localeContext = $localeContext;
        $this->engine = $engine;
    }

    public function getFunctions(): array
    {
        return [
            new TwigFunction('dedi_advertising_render_zone', [$this, 'renderZone'], ['is_safe' => ['html']]),
        ];
    }

    public function renderZone(string $code, ?int $index = null, ?string $template = null): string
    {
        /** @var ChannelInterface $channel */
        $channel = $this->channelContext->getChannel();
        $zone = $this->advertisingZoneRepository->findOneByChannelAndCode($channel, $code, $this->localeContext->getLocaleCode());

        $template = $template ?? '@DediSyliusAdvertisingPlugin/Shop/Zone/index.html.twig';

        return $this->engine->render($template, ['advertising_zone' => $zone, 'index' => $index]);
    }
}
